// =========================================================
// SPINNA.H
//
// spin-edit controls
// =========================================================

#ifndef __SPINEDIT_H
#define __SPINEDIT_H

#if !defined(__OWL_EDIT_H)
  #include <owl\edit.h>
#endif

#if !defined( __CLASSLIB_ARRAYS_H )
  #include <classlib\arrays.h>
#endif

#ifndef __CSTRING_H
  #include <cstring.h>
#endif

#define SPINTOP		1
#define SPINBOTTOM	0

class TSpinner;

typedef float RANGEVALUE;

// =========================================================
// TSPINATTR
// data holder for the spinner
// =========================================================
class TSpinAttr
{
	LPCSTR		lpszFormat;			// format of the spinner
	int			decimals;			// no. of decimal places in lpszFormat
	RANGEVALUE	increment;			// normal button increment
	RANGEVALUE	increment2;			// second increment (SHIFT+key)
	RANGEVALUE	low;				// low range value
	RANGEVALUE	high;				// high range value
	RANGEVALUE	initial;			// initial value
	BOOL		MatchOnLeaving;		// if value should be match to a valid entry
									// when the focus is lost
	BOOL		ShowMenu;			// if menu should be shown on right mouse click

public:
	TSpinAttr ();

	LPCSTR		GetFormat ()						{ return lpszFormat; }
	int			GetDecimals ()						{ return decimals; }
	RANGEVALUE	GetIncrement ()						{ return increment; }
	RANGEVALUE	GetLargeIncrement ()				{ return increment2; }
	RANGEVALUE	GetLow ()							{ return low; }
	RANGEVALUE	GetHigh ()							{ return high; }
	RANGEVALUE	GetInitial ()						{ return initial; }
	BOOL		GetMatchOnLeaving ()				{ return MatchOnLeaving; }
	BOOL		GetShowMenu ()						{ return ShowMenu; }

	void		SetFormat (LPCSTR lpszFormat);
	void		SetIncrement (RANGEVALUE n)			{ increment = n; }
	void		SetLargeIncrement (RANGEVALUE n)	{ increment2 = n; }
	void		SetLow (RANGEVALUE n)				{ low = n; }
	void		SetHigh (RANGEVALUE n)				{ high = n; }
	void		SetInitial (RANGEVALUE n)			{ initial = n; }
	void		SetRange (RANGEVALUE l, RANGEVALUE h);
	void		SetMatchOnLeaving (BOOL b = TRUE)	{ MatchOnLeaving = b; }
	void		SetShowMenu (BOOL m = TRUE)			{ ShowMenu = m; }
};

// =========================================================
// TSPINEDIT
// edit control attached to a spinner
// =========================================================
class TSpinEdit : public TEdit
{
	TSpinner*		spinner;

	void 			Init ();

public:
	TSpinAttr		Spinner;


protected:
	void 			EvChar (UINT key, UINT repeatCount, UINT flags);
	void 			EvKeyDown (UINT key, UINT repeatCount, UINT flags);
	void 			EvKeyUp (UINT key, UINT repeatCount, UINT flags);
	void 			EvKillFocus (HWND hwndGetFocus);
	void 			EvRButtonDown (UINT, TPoint&);
	void			EvEnable (BOOL);

public:
	TSpinEdit (	TWindow*        parent,
				int             id,
				const char far* text,
				int x, int y, int w, int h,
				UINT            textLen = 0,
				TModule*        module = 0);

	TSpinEdit (	TWindow*   		parent,
				int        		resourceId,
				UINT       		textLen = 0,
				TModule*   		module = 0);

	void				EnableButton (BOOL bTop, BOOL bEnable = TRUE);

	virtual string		GetFormat ();
	virtual void		ShowPopup () = 0;
	virtual RANGEVALUE	GetIncrement ()	{ return Spinner.GetIncrement (); }
	virtual void		SpinUp (RANGEVALUE increment = 1) = 0;
	virtual void		SpinDown (RANGEVALUE increment = 1) = 0;
	virtual void		AssessEnable ()	{ }
	virtual void		WhenLeaving ()	{ }

	DECLARE_RESPONSE_TABLE (TSpinEdit);
};

// =========================================================
// TRANGESPINEDIT
// spin-edit which works between two numeric values
// =========================================================
class TRangeSpinEdit : public TSpinEdit
{
public:
	TRangeSpinEdit (TWindow*        parent,
					int             id,
					const char far* text,
					int x, int y, int w, int h,
					UINT            textLen = 0,
					TModule*        module = 0);

	TRangeSpinEdit (TWindow*   		parent,
					int        		resourceId,
					UINT       		textLen = 0,
					TModule*   		module = 0);

	void			SetupWindow ();
	LRESULT			EvCommand (UINT id, HWND hWndCtl, UINT notifyCode);

	RANGEVALUE		GetCurrentValue (BOOL* pValid = NULL);
	void			SetValue (RANGEVALUE nValue, BOOL bSelect = TRUE, BOOL bNotify = TRUE);
	string 			GetValueText (RANGEVALUE nValue);

	virtual void	ShowPopup ();

	virtual void	SpinUp (RANGEVALUE increment = 1);
	virtual void	SpinDown (RANGEVALUE increment = 1);
	virtual void	AssessEnable ();
	virtual void	WhenLeaving ();
};

// =========================================================
// TLISTSPINEDIT
// spin-edit which works from an array of string values
// =========================================================
class TListSpinEdit : public TSpinEdit
{
	TArrayAsVector<string>	arrayString;

public:
	TListSpinEdit (	TWindow*        parent,
					int             id,
					const char far* text,
					int x, int y, int w, int h,
					UINT            textLen = 0,
					TModule*        module = 0);

	TListSpinEdit (	TWindow*   		parent,
					int        		resourceId,
					UINT       		textLen = 0,
					TModule*   		module = 0);

	void			SetupWindow ();
	LRESULT			EvCommand (UINT id, HWND hWndCtl, UINT notifyCode);

	void			AddString (LPCSTR lpszText);

	int				GetCurrentValue (BOOL* pValid = NULL);
	void			SetValue (int nValue, BOOL bSelect = TRUE, BOOL bNotify = TRUE);
	virtual void	ShowPopup ();

	virtual void	SpinUp (RANGEVALUE increment = 1);
	virtual void	SpinDown (RANGEVALUE increment = 1);
	virtual void	AssessEnable ();
	virtual void	WhenLeaving ();
};

#endif
